package vue;

import controleur.ControleurClavier;
import controleur.ControleurEtoile;
import controleur.ControleurSouris;
import controleur.ControleurTrace;
import modele.Dessin;
import modele.Etoile;
import modele.Figure;
import java.util.Iterator;
import modele.Point;
import modele.Trace;

import java.awt.*;
import javax.swing.JFrame;

public class FenetreDeDessin extends JFrame {
    private Dessin dessin;
    private Color couleur;
    private BasicStroke pinceau;
    private ControleurSouris controleurTrace, controleurEtoile, controleurCourant;
    public FenetreDeDessin(ControleurClavier contrClavier, Dessin modele) {
        dessin = modele;
        setTitle("Une première fenêtre pour dessiner à main levée");
        setSize(600, 500);
        setLocationRelativeTo(null);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        controleurTrace = new ControleurTrace(this, dessin);
        controleurEtoile = new ControleurEtoile(this, dessin);
        controleurCourant = controleurTrace;
        addMouseListener(controleurCourant);
        addMouseMotionListener(controleurCourant);
        addKeyListener(contrClavier);
        contrClavier.pour(this);
        changeEpaisseur(1);
        changeCouleur(Color.black);
        setVisible(true);
    }
    public void changeEpaisseur(int ep) {
        pinceau = new BasicStroke(ep, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    }
    public void changeCouleur(Color coul) {
        couleur = coul;
    }
    public void dessineSegment(int x1, int y1, int x2, int y2) {
        Graphics g = getGraphics();
        ((Graphics2D)g).setStroke(pinceau);
        g.setColor(couleur);
        g.drawLine(x1, y1, x2, y2);
    }
    private void changePourControleur(ControleurSouris controleurChoisi) {
        removeMouseListener(controleurCourant);
        removeMouseMotionListener(controleurCourant);
        controleurCourant = controleurChoisi;
        addMouseListener(controleurCourant);
        addMouseMotionListener(controleurCourant);
    }
    public void outilTrace() {
        changePourControleur(controleurTrace);
    }
    public void outilEtoile() {
        changePourControleur(controleurEtoile);
    }
    public void afficheDans(Trace t, Graphics g) {
        Point p1, p2;
        Iterator<Point> it = t.iterator();
        p1 = it.next();
        while(it.hasNext()) {
            p2 = it.next();
            g.drawLine(p1.getX(), p1.getY(), p2.getX(), p2.getY());
            p1 = p2;
        }
    }
    public void afficheDans(Etoile e, Graphics g) {
        Point p1, p2;
        Iterator<Point> it = e.iterator();
        p1 = it.next();
        while(it.hasNext()) {
            p2 = it.next();
            g.drawLine(p1.getX(), p1.getY(), p2.getX(), p2.getY());
        }
    }

    public void paint(Graphics g) {
        g.setColor(Color.white);
        g.fillRect(0, 0, this.getWidth(), this.getHeight());
        ((Graphics2D)g).setStroke(new BasicStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
        g.setColor(Color.black);
        for(Figure f : dessin) {
//        	afficheDans(f, g); //impossible car pas de méthode afficheDans avec un paramètre Figure
        	if (f instanceof Trace) afficheDans((Trace)f, g);
        	if (f instanceof Etoile) afficheDans((Etoile)f, g);
        }
    }
}
